<?php
/**
 *
 *  * This file is part of Boxberry Api.
 *  *
 *  * (c) 2016, T. I. R. Ltd.
 *  * Evgeniy Mosunov, Alexander Borovikov
 *  *
 *  * For the full copyright and license information, please view LICENSE
 *  * file that was distributed with this source code
 *  *
 *  * File: HTTP.php
 *  * Created: 26.07.2016
 *  *
 */

namespace Boxberry\Client;

use Boxberry\Client\Exceptions\BadResponseException;
use Tygh\Exceptions\DeveloperException;
use Tygh\Tools\Url;
use Tygh\Registry;

/**
 * Class HTTP
 * @package Boxberry\Client
 */
class HTTP
{
    public static $lifetime = 3600;

    public static function getCacheTimeLife()
    {
        return self::$lifetime;
    }

    public static function setCacheTimeLife($time)
    {
        self::$lifetime = $time;
    }

    /**
     * @throws DeveloperException
     */
    public static function get_cache($key)
    {
        Registry::registerCache($key, self::getCacheTimeLife(), Registry::cacheLevel('time'));

        if ($cache = Registry::get($key)){
            return $cache;
        }

        Registry::del($key);

        return false;
    }

    public static function set_cache($key, $cnt)
    {
        Registry::set($key, $cnt);
    }

    /**
     * @param $api_url
     * @param $args
     * @return Response
     */
    static function get($api_url, $args)
    {
        return self::send('GET', $api_url, $args);
    }

    /**
     * @param $api_url
     * @param $args
     * @param array $headers
     * @return Response
     * @throws BadResponseException
     */
    static function post($api_url, $args, $headers = [])
    {
        return self::send('POST', $api_url, $args, $headers);
    }

    /**
     * @param $method
     * @param $api_url
     * @param $args
     * @param array $headers
     * @return Response
     * @throws BadResponseException
     */
    private static function send($method, $api_url, $args, $headers = [])
    {
        $url = new Url($api_url);
        $url->setQueryParams($args);
        $url = $url->build();

        $hashRequest = 'boxberry_'.md5($url);

        if ($out = self::get_cache($hashRequest)) {
            $answer = json_decode($out, true);
        } else {
            if ($method === 'GET') {
                $result = \Tygh\Http::get($url);
            } else {
                $result = \Tygh\Http::post($api_url, json_encode($args, JSON_UNESCAPED_UNICODE), $headers);
            }

            if (\Tygh\Http::getStatus() !== 200){
                throw new BadResponseException();
            }

            if (!$answer = json_decode($result, true)) {
                throw new BadResponseException();
            }

            if (isset($answer['err']) && !empty($answer['err'])) {
                throw new BadResponseException($answer['err']);
            }

            if (isset($answer[0]['err'])) {
                throw new BadResponseException($answer[0]['err']);
            }

            if ($method === 'GET') {
                self::set_cache($hashRequest, $result);
            }
        }

        return new Response($answer);
    }

}